@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import argparse
+import filecmp
 import functools
 import operator
 import os
@@ -9,12 +10,14 @@ import re
 import sys
 import uuid
 import warnings
+from difflib import SequenceMatcher
 from functools import singledispatch
 from typing import Iterator
 
 import h5py
 import numpy as np
 import prettytable
+import yaml
 from termcolor import colored
 
 from ._version import version  # noqa: F401
@@ -138,7 +141,8 @@ def getdatapaths(
     :return: List of paths (always absolute, so includes the ``root`` if used).
     """
     kwargs = dict(root=root, max_depth=max_depth, fold=fold, fold_symbol=fold_symbol)
-    return list(getdatasets(file, **kwargs)) + list(getgroups(file, has_attrs=True, **kwargs))
+    ret = list(getdatasets(file, **kwargs)) + list(getgroups(file, has_attrs=True, **kwargs))
+    return list(set(ret))
 
 
 def getgroups(
@@ -618,7 +622,9 @@ def copy(
     :param source_root: Path prefix for all ``source_datasets``.
     :param recursive: If the source is a group, copy all objects within that group recursively.
     :param skip: Skip datasets that are not present in source.
-    :param expand_soft: Copy the underlying data of a link, or copy as link with the same path.
+    :param expand_soft:
+        If True: create a copy of the source dataset.
+        If False: create a link (with identical path as in the source).
     """
 
     if len(source_datasets) == 0:
@@ -678,34 +684,6 @@ def copy(
         )
 
 
-def copydatasets(
-    source: h5py.File,
-    dest: h5py.File,
-    source_datasets: list[str],
-    dest_datasets: list[str] = None,
-    root: str = None,
-):
-    """
-    Copy datasets from one HDF5-archive ``source`` to another HDF5-archive ``dest``.
-    The datasets can be renamed by specifying a list of ``dest_datasets``
-    (whose entries should correspond to the ``source_datasets``).
-    If the source is a Group object,
-    by default all objects within that group will be copied recursively.
-
-    In addition, a ``root`` (path prefix) for the destination datasets name can be specified.
-
-    :param source: The source HDF5-archive.
-    :param dest: The destination HDF5-archive.
-    :param source_datasets: List of dataset-paths in ``source``.
-    :param dest_datasets: List of dataset-paths in ``dest``, defaults to ``source_datasets``.
-    :param root: Path prefix for all ``dest_datasets``.
-    """
-
-    warnings.warn("copydatasets() is deprecated, use copy().", category=DeprecationWarning)
-
-    return copy(source, dest, source_datasets, dest_datasets, root)
-
-
 def isnumeric(a):
     """
     Returns ``True`` is an array contains numeric values.
@@ -725,7 +703,7 @@ def isnumeric(a):
     return False
 
 
-def _equal_value(a, b):
+def _equal_value(a, b, close):
 
     import numpy as np
 
@@ -739,6 +717,8 @@ def _equal_value(a, b):
         return False
 
     if np.issubdtype(a.dtype, np.floating):
+        if close:
+            return np.allclose(a, b)
         if not np.issubdtype(b.dtype, np.floating):
             return False
         if np.allclose(a, b):
@@ -746,6 +726,8 @@ def _equal_value(a, b):
         return False
 
     if np.issubdtype(a.dtype, np.integer):
+        if close:
+            return np.allclose(a, b)
         if not np.issubdtype(b.dtype, np.integer):
             return False
         if np.all(np.equal(a, b)):
@@ -765,10 +747,13 @@ def _equal_value(a, b):
         else:
             return False
 
+    if close:
+        return np.allclose(np.array(list(a[...])), np.array(list(b[...])))
+
     return np.all(np.array(list(a[...])) == np.array(list(b[...])))
 
 
-def _equal(a, b, attrs, matching_dtype, shallow):
+def _equal(a, b, attrs, matching_dtype, shallow, close):
 
     if attrs:
 
@@ -780,7 +765,7 @@ def _equal(a, b, attrs, matching_dtype, shallow):
                     return False
             if shallow:
                 return True
-            if not _equal_value(a.attrs[key], b.attrs[key]):
+            if not _equal_value(a.attrs[key], b.attrs[key], close):
                 return False
 
         for key in b.attrs:
@@ -800,7 +785,7 @@ def _equal(a, b, attrs, matching_dtype, shallow):
     if shallow:
         return True
 
-    return _equal_value(a, b)
+    return _equal_value(a, b, close)
 
 
 def equal(
@@ -812,6 +797,7 @@ def equal(
     attrs: bool = True,
     matching_dtype: bool = False,
     shallow: bool = False,
+    close: bool = False,
 ):
     r"""
     Check that a dataset is equal in both files.
@@ -824,6 +810,7 @@ def equal(
     :param attrs: Compare attributes (the same way at datasets).
     :param matching_dtype: Check that not only the data but also the type matches.
     :param shallow: Check only the presence of the dataset, not its value.
+    :param close: Use ``np.isclose`` also on ``float``-``int`` matches.
     """
 
     if not dest_dataset:
@@ -838,7 +825,7 @@ def equal(
     if dest_dataset not in dest:
         raise OSError(f'"{dest_dataset:s} not in {dest.filename:s}')
 
-    return _equal(source[source_dataset], dest[dest_dataset], attrs, matching_dtype, shallow)
+    return _equal(source[source_dataset], dest[dest_dataset], attrs, matching_dtype, shallow, close)
 
 
 def allequal(
@@ -948,6 +935,7 @@ def compare(
     only_datasets: bool = False,
     fold: str | list[str] = None,
     max_depth: int = None,
+    close: bool = False,
 ) -> dict[list]:
     """
     Compare two files.
@@ -976,6 +964,7 @@ def compare(
     :param max_depth: Set a maximum depth beyond which groups are folded.
     :param fold: Specify groups that are folded.
     :param list_folded: Return folded groups under `"??"`
+    :param close: Use ``np.isclose`` also on ``float``-``int`` matches.
     :return: Dictionary with difference.
     """
     raise NotImplementedError("Overload not found.")
@@ -994,6 +983,7 @@ def _(
     max_depth: int = None,
     fold: str | list[str] = None,
     list_folded: bool = False,
+    close: bool = False,
 ) -> dict[list]:
 
     ret = {"<-": [], "->": [], "!=": [], "==": []}
@@ -1011,8 +1001,10 @@ def _(
     for path in not_in_b:
         ret["->"].append(path)
 
+    opts = dict(attrs=attrs, matching_dtype=matching_dtype, shallow=shallow, close=close)
+
     for path in inboth:
-        if not equal(a, b, path, attrs=attrs, matching_dtype=matching_dtype, shallow=shallow):
+        if not equal(a, b, path, **opts):
             ret["!="].append(path)
         else:
             ret["=="].append(path)
@@ -1036,6 +1028,7 @@ def _(
     max_depth: int = None,
     fold: str | list[str] = None,
     list_folded: bool = False,
+    close: bool = False,
 ) -> dict[list]:
 
     with h5py.File(a, "r") as a_file, h5py.File(b, "r") as b_file:
@@ -1051,6 +1044,7 @@ def _(
             max_depth,
             fold,
             list_folded,
+            close,
         )
 
 
@@ -1068,6 +1062,7 @@ def compare_rename(
     max_depth: int = None,
     fold: str | list[str] = None,
     list_folded: bool = False,
+    close: bool = False,
 ) -> dict[list]:
     """
     Compare two files.
@@ -1114,6 +1109,7 @@ def compare_rename(
     :param max_depth: Set a maximum depth beyond which groups are folded.
     :param fold: Specify groups that are folded.
     :param list_folded: Return folded groups under `"??"`
+    :param close: Use ``np.isclose`` also on ``float``-``int`` matches.
     :return: Dictionary with difference.
     """
 
@@ -1123,6 +1119,8 @@ def compare_rename(
         shallow=shallow,
         max_depth=max_depth,
         fold=fold,
+        list_folded=list_folded,
+        close=close,
     )
 
     if rename is None:
@@ -1164,6 +1162,7 @@ def compare_rename(
 
     opts.pop("max_depth")
     opts.pop("fold")
+    opts.pop("list_folded")
 
     for path_a, path_b in zip(rename_a, rename_b):
         if not equal(a, b, path_a, path_b, **opts):
@@ -1464,6 +1463,7 @@ def G5list(args: list[str]):
 
     parser = _G5list_parser()
     args = parser.parse_args(args)
+    opts = dict(root=args.root, max_depth=args.max_depth, fold=args.fold)
 
     if not os.path.isfile(args.source):
         raise OSError(f'"{args.source}" does not exist')
@@ -1473,11 +1473,9 @@ def G5list(args: list[str]):
         if args.layer is not None:
             paths = sorted(join(args.layer, i, root=True) for i in source[args.layer])
         else:
-            paths = list(
-                getdatasets(source, root=args.root, max_depth=args.max_depth, fold=args.fold)
-            )
+            paths = list(getdatasets(source, **opts))
             if not args.datasets:
-                paths += getgroups(source, root=args.root, has_attrs=True, max_depth=args.max_depth)
+                paths += getgroups(source, has_attrs=True, **opts)
             paths = sorted(list(set(paths)))
 
         if args.min_attrs is not None:
@@ -1518,6 +1516,9 @@ def _G5compare_parser():
     parser.add_argument(
         "-t", "--dtype", action="store_true", help="Verify that the type of the datasets match."
     )
+    parser.add_argument(
+        "--close", action="store_true", help="Use ``np.isclose`` also on ``float``-``int`` matches."
+    )
     parser.add_argument(
         "--shallow",
         action="store_true",
@@ -1527,60 +1528,16 @@ def _G5compare_parser():
     parser.add_argument("-d", "--max-depth", type=int, help="Maximum depth to display")
     parser.add_argument("-f", "--fold", action="append", help="Fold paths")
     parser.add_argument("-r", "--renamed", nargs=2, action="append", help="Renamed paths.")
+    parser.add_argument("-R", "--renamed-yaml", type=str, help="YAML file with renamed paths.")
     parser.add_argument("-c", "--colors", default="dark", help="Color theme: dark/light/none")
     parser.add_argument("--table", default="SINGLE_BORDER", help="Table theme")
+    parser.add_argument("-I", "--input", action="store_true", help="Header identical to input")
     parser.add_argument("-v", "--version", action="version", version=version)
     parser.add_argument("a", help="Path to HDF5-file.")
     parser.add_argument("b", help="Path to HDF5-file.")
     return parser
 
 
-def _truncate_print_path(path: str, n: int) -> str:
-    """
-    Truncate a file path, such that the printed length is smaller than ``n``.
-    For example::
-
-        "/path/to/long/foo/bar" -> "/path/.../bar"
-
-    :param path: Path to truncate.
-    :param n: Maximum length of the path.
-    :return: Truncated path.
-    """
-
-    if len(path) <= n:
-        return path
-
-    path = os.path.abspath(path).split(os.sep)
-    if len(path[0]) == 0:
-        path[0] = os.sep
-
-    length = np.array([len(i) for i in path])
-    incl = np.zeros(len(path), dtype=bool)
-    incl[0] = True
-    incl[-1] = True
-
-    # try including from the back
-    for i in range(1, len(path) - 1)[::-1]:
-        incl[i] = True
-        if np.sum(length[incl]) + np.sum(incl) + 3 > n:
-            incl[i] = False
-            break
-
-    # try including from the front
-    for i in range(1, len(path) - 1):
-        incl[i] = True
-        if np.sum(length[incl]) + np.sum(incl) + 3 > n:
-            incl[i] = False
-            break
-
-    if np.sum(incl) != len(path):
-        i = np.argmin(incl)
-        j = i + np.argmax(incl[i:])
-        path = path[:i] + ["..."] + path[j:]
-
-    return os.path.join(*path)
-
-
 def G5compare(args: list[str]):
     """
     Command-line tool to print datasets from a file, see ``--help``.
@@ -1594,6 +1551,18 @@ def G5compare(args: list[str]):
         if not os.path.isfile(filepath):
             raise OSError(f'"{filepath}" does not exist')
 
+    if filecmp.cmp(args.a, args.b):
+        print("Files are identical")
+        return 0
+
+    if args.renamed_yaml is not None:
+        with open(args.renamed_yaml) as file:
+            ret = yaml.load(file.read(), Loader=yaml.FullLoader)
+        assert type(ret) is dict
+        if args.renamed is None:
+            args.renamed = []
+        args.renamed += [[key, ret[key]] for key in ret]
+
     with h5py.File(args.a, "r") as a, h5py.File(args.b, "r") as b:
         comp, r_a, r_b = compare_rename(
             a,
@@ -1606,13 +1575,9 @@ def G5compare(args: list[str]):
             fold=args.fold,
             max_depth=args.max_depth,
             list_folded=True,
+            close=args.close,
         )
 
-    def print_path(path):
-        if len(os.path.relpath(path).split("..")) < 3:
-            return os.path.relpath(path)
-        return os.path.abspath(path)
-
     def def_row(arg, colors):
 
         if colors == "none":
@@ -1655,33 +1620,35 @@ def G5compare(args: list[str]):
     for path_a, path_b in zip(r_a["!="], r_b["!="]):
         out.add_row(def_row([path_a, "!=", path_b], args.colors))
 
-    file_a = print_path(args.a)
-    file_b = print_path(args.b)
-    n = max(len(row[0]) for row in out.rows)
-    out.field_names = [_truncate_print_path(file_a, n), "", _truncate_print_path(file_b, n)]
+    if len(out.rows) == 0:
+        print("No differences found")
+        return
 
+    cols = [args.a, args.b]
+
+    if not args.input:
+        sizes = [max(len(row[0]) for row in out.rows), max(len(row[2]) for row in out.rows)]
+
+        if (len(cols[0]) > sizes[0] or len(cols[1]) > sizes[1]) and (
+            len(cols[0]) + len(cols[1]) > 90
+        ):
+            trial = [os.path.abspath(i) for i in cols]
+            s = SequenceMatcher(None, *trial).get_matching_blocks()[0].size
+            for i in range(2):
+                n = len(trial[i][s:].split(os.sep))
+                cols[i] = os.path.join(*(["..."] + trial[i].split(os.sep)[-n:]))
+
+    out.field_names = [cols[0], "", cols[1]]
     print(out.get_string())
 
 
-def _G5print_catch():
-    try:
-        G5print(sys.argv[1:])
-    except Exception as e:
-        print(e)
-        return 1
+def _G5print_cli():
+    G5print(sys.argv[1:])
 
 
-def _G5list_catch():
-    try:
-        G5list(sys.argv[1:])
-    except Exception as e:
-        print(e)
-        return 1
+def _G5list_cli():
+    G5list(sys.argv[1:])
 
 
-def _G5compare_catch():
-    try:
-        G5compare(sys.argv[1:])
-    except Exception as e:
-        print(e)
-        return 1
+def _G5compare_cli():
+    G5compare(sys.argv[1:])
